@@ -14,6 +14,7 @@
 #include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/uri.hpp>
 
+#include <assert.h>
 #include <string.h>
 
 int simple_body::data_request(unsigned char* data, unsigned int & len)
@@ -105,6 +106,63 @@ int HttpResponse::reset()
 	return FZ_REPLY_CONTINUE;
 }
 
+void RequestThrottler::throttle(std::string const& hostname, fz::datetime const& backoff)
+{
+	if (hostname.empty() || !backoff) {
+		return;
+	}
+
+	fz::scoped_lock l(mtx_);
+
+	bool found{};
+	auto now = fz::datetime::now();
+	for (size_t i = 0; i < backoff_.size(); ) {
+		auto & entry = backoff_[i];
+		if (entry.first == hostname) {
+			found = true;
+			if (entry.second < backoff) {
+				entry.second = backoff;
+			}
+		}
+		if (entry.second < now) {
+			backoff_[i] = std::move(backoff_.back());
+			backoff_.pop_back();
+		}
+		else {
+			++i;
+		}
+	}
+	if (!found) {
+		backoff_.emplace_back(hostname, backoff);
+	}
+}
+
+fz::duration RequestThrottler::get_throttle(std::string const& hostname)
+{
+	fz::scoped_lock l(mtx_);
+
+	fz::duration ret;
+
+	auto now = fz::datetime::now();
+	for (size_t i = 0; i < backoff_.size(); ) {
+		auto & entry = backoff_[i];
+		if (entry.second < now) {
+			backoff_[i] = std::move(backoff_.back());
+			backoff_.pop_back();
+		}
+		else {
+			if (entry.first == hostname) {
+				ret = entry.second - now;
+			}
+			++i;
+		}
+	}
+
+	return ret;
+}
+
+RequestThrottler CHttpControlSocket::throttler_;
+
 CHttpControlSocket::CHttpControlSocket(CFileZillaEnginePrivate & engine)
 	: CRealControlSocket(engine)
 {
