@@ -101,26 +101,6 @@ std::wstring CServer::GetUser() const
 	return m_user;
 }
 
-CServer& CServer::operator=(const CServer &op)
-{
-	m_protocol = op.m_protocol;
-	m_type = op.m_type;
-	m_host = op.m_host;
-	m_port = op.m_port;
-	m_user = op.m_user;
-	m_timezoneOffset = op.m_timezoneOffset;
-	m_pasvMode = op.m_pasvMode;
-	m_maximumMultipleConnections = op.m_maximumMultipleConnections;
-	m_encodingType = op.m_encodingType;
-	m_customEncoding = op.m_customEncoding;
-	m_postLoginCommands = op.m_postLoginCommands;
-	m_bypassProxy = op.m_bypassProxy;
-	m_name = op.m_name;
-	extraParameters_ = op.extraParameters_;
-
-	return *this;
-}
-
 bool CServer::operator==(const CServer &op) const
 {
 	if (m_protocol != op.m_protocol) {
@@ -268,7 +248,7 @@ CServer::CServer(ServerProtocol protocol, ServerType type, std::wstring const& h
 		m_port = port;
 	}
 	else {
-		port = GetDefaultPort(protocol);
+		m_port = GetDefaultPort(protocol);
 	}
 }
 
@@ -288,7 +268,7 @@ void CServer::SetProtocol(ServerProtocol serverProtocol)
 	m_protocol = serverProtocol;
 
 	// Clear out parameters not supported by the current protocol
-	std::map<std::string, std::wstring> oldParams;
+	std::map<std::string, std::wstring, std::less<>> oldParams;
 	std::swap(extraParameters_, oldParams);
 	for (auto const& param : oldParams) {
 		SetExtraParameter(param.first, param.second);
@@ -437,18 +417,6 @@ bool CServer::SetEncodingType(CharsetEncoding type, std::wstring const& encoding
 	return true;
 }
 
-bool CServer::SetCustomEncoding(std::wstring const& encoding)
-{
-	if (encoding.empty()) {
-		return false;
-	}
-
-	m_encodingType = ENCODING_CUSTOM;
-	m_customEncoding = encoding;
-
-	return true;
-}
-
 CharsetEncoding CServer::GetEncodingType() const
 {
 	return m_encodingType;
@@ -615,7 +583,7 @@ bool CServer::ProtocolHasFeature(ServerProtocol const protocol, ProtocolFeature
 		}
 		break;
 	case ProtocolFeature::RecursiveDelete:
-		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE) {
+		if (protocol == GOOGLE_DRIVE || protocol == DROPBOX || protocol == ONEDRIVE || protocol == B2) {
 			return true;
 		}
 		break;
@@ -664,7 +632,7 @@ void CServer::ClearExtraParameters()
 	extraParameters_.clear();
 }
 
-std::wstring CServer::GetExtraParameter(std::string const& name) const
+std::wstring CServer::GetExtraParameter(std::string_view const& name) const
 {
 	auto it = extraParameters_.find(name);
 	if (it != extraParameters_.cend()) {
@@ -673,19 +641,27 @@ std::wstring CServer::GetExtraParameter(std::string const& name) const
 	return std::wstring();
 }
 
-std::map<std::string, std::wstring> const& CServer::GetExtraParameters() const
+std::map<std::string, std::wstring, std::less<>> const& CServer::GetExtraParameters() const
 {
 	return extraParameters_;
 }
 
-void CServer::SetExtraParameter(std::string const& name, std::wstring const& value)
+bool CServer::HasExtraParameter(std::string_view const& name) const
 {
+	return extraParameters_.find(name) != extraParameters_.cend();
+}
+
+void CServer::SetExtraParameter(std::string_view const& name, std::wstring const& value)
+{
+	auto it = extraParameters_.find(name);
 	if (value.empty()) {
-		extraParameters_.erase(name);
+		if (it != extraParameters_.cend()) {
+			extraParameters_.erase(it);
+		}
 	}
 	else {
 		bool found = false;
-		auto const& traits  = ExtraServerParameterTraits(m_protocol);
+		auto const& traits = ExtraServerParameterTraits(m_protocol);
 		for (auto const& trait : traits) {
 			if (trait.section_ != ParameterSection::credentials && name == trait.name_) {
 				found = true;
@@ -694,14 +670,22 @@ void CServer::SetExtraParameter(std::string const& name, std::wstring const& val
 		}
 
 		if (found) {
-			extraParameters_[name] = value;
+			if (it == extraParameters_.cend()) {
+				extraParameters_.emplace(name, value);
+			}
+			else {
+				it->second = value;
+			}
 		}
 	}
 }
 
-void CServer::ClearExtraParameter(const std::string &name)
+void CServer::ClearExtraParameter(std::string_view const& name)
 {
-	extraParameters_.erase(name);
+	auto it = extraParameters_.find(name);
+	if (it != extraParameters_.cend()) {
+		extraParameters_.erase(it);
+	}
 }
 
 LogonType GetLogonTypeFromName(std::wstring const& name)
@@ -769,7 +753,7 @@ void Credentials::ClearExtraParameters()
 	extraParameters_.clear();
 }
 
-std::wstring Credentials::GetExtraParameter(std::string const& name) const
+std::wstring Credentials::GetExtraParameter(std::string_view const& name) const
 {
 	auto it = extraParameters_.find(name);
 	if (it != extraParameters_.cend()) {
@@ -778,24 +762,42 @@ std::wstring Credentials::GetExtraParameter(std::string const& name) const
 	return std::wstring();
 }
 
-std::map<std::string, std::wstring> const& Credentials::GetExtraParameters() const
+std::map<std::string, std::wstring, std::less<>> const& Credentials::GetExtraParameters() const
 {
 	return extraParameters_;
 }
 
-void Credentials::SetExtraParameter(ServerProtocol protocol, std::string const& name, std::wstring const& value)
+bool Credentials::HasExtraParameter(std::string_view const& name) const
 {
-	bool found = false;
-	auto const& traits = ExtraServerParameterTraits(protocol);
-	for (auto const& trait : traits) {
-		if (trait.section_ != ParameterSection::credentials && name == trait.name_) {
-			found = true;
-			break;
+	return extraParameters_.find(name) != extraParameters_.cend();
+}
+
+void Credentials::SetExtraParameter(ServerProtocol protocol, std::string_view const& name, std::wstring const& value)
+{
+	auto it = extraParameters_.find(name);
+	if (value.empty()) {
+		if (it != extraParameters_.cend()) {
+			extraParameters_.erase(it);
 		}
 	}
+	else {
+		bool found = false;
+		auto const& traits = ExtraServerParameterTraits(protocol);
+		for (auto const& trait : traits) {
+			if (trait.section_ != ParameterSection::credentials && name == trait.name_) {
+				found = true;
+				break;
+			}
+		}
 
-	if (found) {
-		extraParameters_[name] = value;
+		if (found) {
+			if (it == extraParameters_.cend()) {
+				extraParameters_.emplace(name, value);
+			}
+			else {
+				it->second = value;
+			}
+		}
 	}
 }
 
@@ -843,7 +845,8 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 		{
 			static std::vector<ParameterTraits> ret = []() {
 				std::vector<ParameterTraits> ret;
-				ret.emplace_back(ParameterTraits{"email", ParameterSection::user, ParameterTraits::optional, std::wstring(), std::wstring()});
+				ret.emplace_back(ParameterTraits{"login_hint", ParameterSection::user, ParameterTraits::optional, std::wstring(), _("Name or email address")});
+				ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
 				return ret;
 			}();
 			return ret;
@@ -869,6 +872,46 @@ std::vector<ParameterTraits> const& ExtraServerParameterTraits(ServerProtocol pr
 			}();
 			return ret;
 		}
+	case GOOGLE_DRIVE:
+	{
+		static std::vector<ParameterTraits> ret = []() {
+			std::vector<ParameterTraits> ret;
+			ret.emplace_back(ParameterTraits{"login_hint", ParameterSection::user, ParameterTraits::optional, std::wstring(), _("Name or email address")});
+			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			return ret;
+		}();
+		return ret;
+	}
+	case DROPBOX:
+	{
+		static std::vector<ParameterTraits> ret = []() {
+			std::vector<ParameterTraits> ret;
+			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			return ret;
+		}();
+		return ret;
+	}
+	case ONEDRIVE:
+	{
+		static std::vector<ParameterTraits> ret = []() {
+			std::vector<ParameterTraits> ret;
+			ret.emplace_back(ParameterTraits{"login_hint", ParameterSection::user, ParameterTraits::optional, std::wstring(), _("Name or email address")});
+			ret.emplace_back(ParameterTraits{"nowebview", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			return ret;
+		}();
+		return ret;
+	}
+	case BOX:
+	{
+		static std::vector<ParameterTraits> ret = []() {
+			std::vector<ParameterTraits> ret;
+			ret.emplace_back(ParameterTraits{"nowebview", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			ret.emplace_back(ParameterTraits{"oauth_identity", ParameterSection::custom, ParameterTraits::optional, std::wstring(), std::wstring()});
+			return ret;
+		}();
+		return ret;
+	}
 	default:
 		break;
 	}
@@ -908,7 +951,7 @@ std::tuple<std::wstring, std::wstring> GetDefaultHost(ServerProtocol protocol)
 
 bool ProtocolHasUser(ServerProtocol protocol)
 {
-	return protocol != DROPBOX && protocol != ONEDRIVE && protocol != BOX;
+	return protocol != DROPBOX && protocol != ONEDRIVE && protocol != BOX && protocol != GOOGLE_DRIVE;
 }
 
 bool CServer::SameResource(CServer const& other) const
@@ -933,3 +976,22 @@ bool CServer::SameContent(CServer const& other) const
 
 	return l == r;
 }
+
+CaseSensitivity GetCaseSensitivity(ServerProtocol protocol)
+{
+	switch (protocol) {
+	case B2:
+	case GOOGLE_DRIVE:
+		return CaseSensitivity::yes;
+	case BOX:
+	case ONEDRIVE:
+		return CaseSensitivity::no;
+	default:
+		return CaseSensitivity::unspecified;
+	}
+}
+
+CaseSensitivity CServer::GetCaseSensitivity() const
+{
+	return ::GetCaseSensitivity(m_protocol);
+}
