@@ -9,12 +9,14 @@
 #define PACKAGE_STRING "FileZilla 3"
 #endif
 
-#include <wx/defs.h>
+#include <libfilezilla/libfilezilla.hpp>
 
-// Include after defs.h so that __WXFOO__ is properly defined
 #include "setup.h"
 
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
+#ifndef UNICODE
+#define UNICODE
+#endif
 #ifndef NOMINMAX
 #define NOMINMAX
 #endif
@@ -24,18 +26,6 @@
 #include <windows.h>
 #endif
 
-#include <wx/file.h>
-#include <wx/string.h>
-#include <wx/translation.h>
-
-#include <list>
-#include <vector>
-#include <map>
-
-#include <libfilezilla/libfilezilla.hpp>
-
-#include <libfilezilla/glue/wx.hpp>
-
 #include "optionsbase.h"
 #include "logging.h"
 #include "server.h"
@@ -47,6 +37,4 @@
 
 #include "misc.h"
 
-#define TRANSLATE_T(str) _T(str)
-
 #endif
