@@ -33,7 +33,7 @@
 #include "osx_sandbox_userdirs.h"
 #endif
 
-#ifndef __WXGTK__
+#if !defined(__WXGTK__) && !defined(__MINGW32__)
 IMPLEMENT_APP(CFileZillaApp)
 #else
 IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
@@ -212,6 +212,11 @@ bool CFileZillaApp::OnInit()
 {
 	AddStartupProfileRecord("CFileZillaApp::OnInit()");
 
+	// Turn off idle events, we don't need them
+	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
+
+	wxUpdateUIEvent::SetMode(wxUPDATE_UI_PROCESS_SPECIFIED);
+
 	fz::set_translators(translator, translator_pf);
 
 #ifdef __WXMSW__
@@ -289,22 +294,16 @@ USE AT OWN RISK"), _T("Important Information"));
 		return false;
 	}
 
+	themeProvider_ = std::make_unique<CThemeProvider>();
 	CheckExistsFzsftp();
 #if ENABLE_STORJ
 	CheckExistsFzstorj();
 #endif
 
-	// Turn off idle events, we don't need them
-	wxIdleEvent::SetMode(wxIDLE_PROCESS_SPECIFIED);
-
-	wxUpdateUIEvent::SetMode(wxUPDATE_UI_PROCESS_SPECIFIED);
-
 #ifdef WITH_LIBDBUS
 	CSessionManager::Init();
 #endif
 
-	themeProvider_ = std::make_unique<CThemeProvider>();
-
 	// Load the text wrapping engine
 	m_pWrapEngine = std::make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
@@ -313,14 +312,14 @@ USE AT OWN RISK"), _T("Important Information"));
 #ifdef USE_MAC_SANDBOX
 	OSXSandboxUserdirs::Get().Load();
 
-    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+	if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
 		CWelcomeDialog welcome;
 		welcome.Run(nullptr, false);
 		welcome_skip = true;
 
-        OSXSandboxUserdirsDialog dlg;
-        dlg.Run(nullptr, true);
-	    if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
+		OSXSandboxUserdirsDialog dlg;
+		dlg.Run(nullptr, true);
+		if (OSXSandboxUserdirs::Get().GetDirs().empty()) {
 			return false;
 		}
     }
@@ -471,8 +470,9 @@ bool CFileZillaApp::LoadResourceFiles()
 		wxMessageBoxEx(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
 		return false;
 	}
-
-	m_resourceDir.AddSegment(_T("resources"));
+	else {
+		m_resourceDir.AddSegment(_T("resources"));
+	}
 
 	// Useful for XRC files with embedded image data.
 	wxFileSystem::AddHandler(new wxFileSystemBlobHandler);
@@ -795,3 +795,15 @@ std::wstring CFileZillaApp::GetSettingsFile(std::wstring const& name) const
 {
 	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
 }
+
+#if defined(__MINGW32__)
+extern "C" {
+__declspec(dllexport) // This forces ld to not strip relocations so that ASLR can work on MSW.
+int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR, int nCmdShow)
+{
+	wxDISABLE_DEBUG_SUPPORT();
+	return wxEntry(hInstance, hPrevInstance, NULL, nCmdShow);
+}
+}
+#endif
+
