@@ -421,7 +421,7 @@ void CQueueView::ProcessNotification(t_EngineData* pEngineData, std::unique_ptr<
 	switch (pNotification->GetID())
 	{
 	case nId_logmsg:
-		m_pMainFrame->GetStatusView()->AddToLog(static_cast<CLogmsgNotification&>(*pNotification.get()));
+		m_pMainFrame->GetStatusView()->AddToLog(std::move(static_cast<CLogmsgNotification&>(*pNotification.get())));
 		if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2) {
 			m_pQueue->Highlight(3);
 		}
@@ -1248,16 +1248,6 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 				return;
 			}
 
-			if (res == FZ_REPLY_NOTCONNECTED) {
-				if (engineData.transient) {
-					ResetEngine(engineData, ResetReason::retry);
-					return;
-				}
-
-				engineData.state = t_EngineData::connect;
-				continue;
-			}
-
 			if (res == FZ_REPLY_OK) {
 				ResetEngine(engineData, ResetReason::success);
 				return;
@@ -1282,23 +1272,20 @@ void CQueueView::SendNextCommand(t_EngineData& engineData)
 				return;
 			}
 
-			if (res == FZ_REPLY_NOTCONNECTED) {
-				if (engineData.transient) {
-					ResetEngine(engineData, ResetReason::retry);
-					return;
-				}
-
-				engineData.state = t_EngineData::connect;
-				continue;
-			}
-
 			if (res == FZ_REPLY_OK) {
 				ResetEngine(engineData, ResetReason::success);
 				return;
 			}
 
-			// Pointless to retry
-			ResetEngine(engineData, ResetReason::failure);
+			if (res & FZ_REPLY_DISCONNECTED) {
+				if (IncreaseErrorCount(engineData)) {
+					continue;
+				}
+			}
+			else {
+				// Pointless to retry
+				ResetEngine(engineData, ResetReason::failure);
+			}
 			return;
 		}
 	}
