@@ -14,7 +14,7 @@ BEGIN_EVENT_TABLE(wxDialogEx, wxDialog)
 EVT_CHAR_HOOK(wxDialogEx::OnChar)
 END_EVENT_TABLE()
 
-int wxDialogEx::m_shown_dialogs = 0;
+std::vector<wxDialogEx*> wxDialogEx::shown_dialogs_;
 
 #ifdef __WXMAC__
 static int const pasteId = wxNewId();
@@ -24,7 +24,7 @@ extern wxTextEntry* GetSpecialTextEntry(wxWindow*, wxChar);
 
 bool wxDialogEx::ProcessEvent(wxEvent& event)
 {
-	if(event.GetEventType() != wxEVT_MENU) {
+	if (event.GetEventType() != wxEVT_MENU) {
 		return wxDialog::ProcessEvent(event);
 	}
 
@@ -43,6 +43,17 @@ bool wxDialogEx::ProcessEvent(wxEvent& event)
 }
 #endif
 
+bool wxDialogEx::Create(wxWindow* parent, int id, wxString const& title, wxPoint const& pos, wxSize const& size, long style)
+{
+	bool ret = wxDialog::Create(parent, id, title, pos, size, style);
+#ifdef __WXMAC__
+	if (ret) {
+		FixPasswordPaste(acceleratorTable_);
+	}
+#endif
+	return ret;
+}
+
 void wxDialogEx::OnChar(wxKeyEvent& event)
 {
 	if (event.GetKeyCode() == WXK_ESCAPE) {
@@ -63,12 +74,8 @@ bool wxDialogEx::Load(wxWindow* pParent, wxString const& name, std::wstring cons
 		return false;
 	}
 
-#ifdef __WXMAC__
-	wxAcceleratorEntry entries[2];
-	entries[0].Set(wxACCEL_CMD, 'V', pasteId);
-	entries[1].Set(wxACCEL_CMD, 'A', selectAllId);
-	wxAcceleratorTable accel(sizeof(entries) / sizeof(wxAcceleratorEntry), entries);
-	SetAcceleratorTable(accel);
+#ifdef __WCMAC__
+	FixPasswordPaste(acceleratorTable_);
 #endif
 
 	return true;
@@ -121,11 +128,18 @@ int wxDialogEx::ShowModal()
 	::ReleaseCapture();
 #endif
 
-	++m_shown_dialogs;
+	shown_dialogs_.push_back(this);
+
+	if (acceleratorTable_.empty()) {
+		SetAcceleratorTable(wxNullAcceleratorTable);
+	}
+	else {
+		SetAcceleratorTable(wxAcceleratorTable(acceleratorTable_.size(), acceleratorTable_.data()));
+	}
 
 	int ret = wxDialog::ShowModal();
 
-	--m_shown_dialogs;
+	shown_dialogs_.pop_back();
 
 	return ret;
 }
@@ -140,10 +154,15 @@ bool wxDialogEx::ReplaceControl(wxWindow* old, wxWindow* wnd)
 	return true;
 }
 
-bool wxDialogEx::CanShowPopupDialog()
+bool wxDialogEx::CanShowPopupDialog(wxTopLevelWindow * parent)
 {
-	if (m_shown_dialogs != 0 || IsShowingMessageBox()) {
-		// There already is a dialog or message box showing
+	if (IsShowingMessageBox()) {
+		// There already a message box showing
+		return false;
+	}
+
+	if (!shown_dialogs_.empty() && shown_dialogs_.back() != parent) {
+		// There is an open dialog which isn't the expected parent
 		return false;
 	}
 
@@ -203,6 +222,7 @@ wxSizerFlags const DialogLayout::grow(wxSizerFlags().Expand());
 wxSizerFlags const DialogLayout::valign(wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL));
 wxSizerFlags const DialogLayout::halign(wxSizerFlags().Align(wxALIGN_CENTER_HORIZONTAL));
 wxSizerFlags const DialogLayout::valigng(wxSizerFlags().Align(wxALIGN_CENTER_VERTICAL).Expand());
+wxSizerFlags const DialogLayout::ralign(wxSizerFlags().Align(wxALIGN_RIGHT));
 
 DialogLayout::DialogLayout(wxTopLevelWindow * parent)
 	: parent_(parent)
@@ -233,6 +253,12 @@ wxFlexGridSizer* DialogLayout::createFlex(int cols, int rows) const
 	return new wxFlexGridSizer(rows, cols, g, g);
 }
 
+wxGridSizer* DialogLayout::createGrid(int cols, int rows) const
+{
+	int const g = gap;
+	return new wxGridSizer(rows, cols, g, g);
+}
+
 wxGridBagSizer* DialogLayout::createGridBag(int cols, int rows) const
 {
 	int const g = gap;
@@ -294,7 +320,27 @@ wxSizerItem* DialogLayout::gbAdd(wxGridBagSizer* gb, wxSizer* sizer, wxSizerFlag
 	return item;
 }
 
+std::tuple<wxStaticBox*, wxFlexGridSizer*> DialogLayout::createStatBox(wxSizer* parent, wxString const& title, int cols, int rows) const
+{
+	auto* boxSizer = new wxStaticBoxSizer(wxHORIZONTAL, parent->GetContainingWindow(), title);
+	auto* box = boxSizer->GetStaticBox();
+	parent->Add(boxSizer, 1, wxGROW);
+
+	auto* flex = createFlex(cols, rows);
+	boxSizer->Add(flex, 1, wxALL|wxGROW, border);
+
+	return std::make_tuple(box, flex);
+}
+
 std::wstring LabelEscape(std::wstring const& label)
 {
 	return fz::replaced_substrings(label, L"&", L"&&");
 }
+
+#ifdef __WXMAC__
+void FixPasswordPaste(std::vector<wxAcceleratorEntry> & entries)
+{
+	entries.emplace_back(wxACCEL_CMD, 'V', pasteId);
+	entries.emplace_back(wxACCEL_CMD, 'A', selectAllId);
+}
+#endif
